# Copyright (c) 2018-2019 Phase Advanced Sensor Systems, Inc.
from ..device import Device, Reg32, Reg32R, Reg32W


class GPIO(Device):
    '''
    Driver for a GPIO Port device.
    '''
    REGS = [Reg32 ('MODER',     0x00,  [('MODE0',   2),
                                        ('MODE1',   2),
                                        ('MODE2',   2),
                                        ('MODE3',   2),
                                        ('MODE4',   2),
                                        ('MODE5',   2),
                                        ('MODE6',   2),
                                        ('MODE7',   2),
                                        ('MODE8',   2),
                                        ('MODE9',   2),
                                        ('MODE10',  2),
                                        ('MODE11',  2),
                                        ('MODE12',  2),
                                        ('MODE13',  2),
                                        ('MODE14',  2),
                                        ('MODE15',  2),
                                        ]),
            Reg32 ('OTYPER',    0x04,  [('OT0',     1),
                                        ('OT1',     1),
                                        ('OT2',     1),
                                        ('OT3',     1),
                                        ('OT4',     1),
                                        ('OT5',     1),
                                        ('OT6',     1),
                                        ('OT7',     1),
                                        ('OT8',     1),
                                        ('OT9',     1),
                                        ('OT10',    1),
                                        ('OT11',    1),
                                        ('OT12',    1),
                                        ('OT13',    1),
                                        ('OT14',    1),
                                        ('OT15',    1),
                                        ]),
            Reg32 ('OSPEEDR',   0x08,  [('OSPEED0',     2),
                                        ('OSPEED1',     2),
                                        ('OSPEED2',     2),
                                        ('OSPEED3',     2),
                                        ('OSPEED4',     2),
                                        ('OSPEED5',     2),
                                        ('OSPEED6',     2),
                                        ('OSPEED7',     2),
                                        ('OSPEED8',     2),
                                        ('OSPEED9',     2),
                                        ('OSPEED10',    2),
                                        ('OSPEED11',    2),
                                        ('OSPEED12',    2),
                                        ('OSPEED13',    2),
                                        ('OSPEED14',    2),
                                        ('OSPEED15',    2),
                                        ]),
            Reg32 ('PUPDR',     0x0C,  [('PUPD0',       2),
                                        ('PUPD1',       2),
                                        ('PUPD2',       2),
                                        ('PUPD3',       2),
                                        ('PUPD4',       2),
                                        ('PUPD5',       2),
                                        ('PUPD6',       2),
                                        ('PUPD7',       2),
                                        ('PUPD8',       2),
                                        ('PUPD9',       2),
                                        ('PUPD10',      2),
                                        ('PUPD11',      2),
                                        ('PUPD12',      2),
                                        ('PUPD13',      2),
                                        ('PUPD14',      2),
                                        ('PUPD15',      2),
                                        ]),
            Reg32R('IDR',       0x10,  [('ID0',         1),
                                        ('ID1',         1),
                                        ('ID2',         1),
                                        ('ID3',         1),
                                        ('ID4',         1),
                                        ('ID5',         1),
                                        ('ID6',         1),
                                        ('ID7',         1),
                                        ('ID8',         1),
                                        ('ID9',         1),
                                        ('ID10',        1),
                                        ('ID11',        1),
                                        ('ID12',        1),
                                        ('ID13',        1),
                                        ('ID14',        1),
                                        ('ID15',        1),
                                        ]),
            Reg32 ('ODR',       0x14,  [('OD0',         1),
                                        ('OD1',         1),
                                        ('OD2',         1),
                                        ('OD3',         1),
                                        ('OD4',         1),
                                        ('OD5',         1),
                                        ('OD6',         1),
                                        ('OD7',         1),
                                        ('OD8',         1),
                                        ('OD9',         1),
                                        ('OD10',        1),
                                        ('OD11',        1),
                                        ('OD12',        1),
                                        ('OD13',        1),
                                        ('OD14',        1),
                                        ('OD15',        1),
                                        ]),
            Reg32W('BSRR',      0x18,  [('BS0',         1),
                                        ('BS1',         1),
                                        ('BS2',         1),
                                        ('BS3',         1),
                                        ('BS4',         1),
                                        ('BS5',         1),
                                        ('BS6',         1),
                                        ('BS7',         1),
                                        ('BS8',         1),
                                        ('BS9',         1),
                                        ('BS10',        1),
                                        ('BS11',        1),
                                        ('BS12',        1),
                                        ('BS13',        1),
                                        ('BS14',        1),
                                        ('BS15',        1),
                                        ('BR0',         1),
                                        ('BR1',         1),
                                        ('BR2',         1),
                                        ('BR3',         1),
                                        ('BR4',         1),
                                        ('BR5',         1),
                                        ('BR6',         1),
                                        ('BR7',         1),
                                        ('BR8',         1),
                                        ('BR9',         1),
                                        ('BR10',        1),
                                        ('BR11',        1),
                                        ('BR12',        1),
                                        ('BR13',        1),
                                        ('BR14',        1),
                                        ('BR15',        1),
                                        ]),
            Reg32 ('LCKR',      0x1C,  [('LCK0',        1),
                                        ('LCK1',        1),
                                        ('LCK2',        1),
                                        ('LCK3',        1),
                                        ('LCK4',        1),
                                        ('LCK5',        1),
                                        ('LCK6',        1),
                                        ('LCK7',        1),
                                        ('LCK8',        1),
                                        ('LCK9',        1),
                                        ('LCK10',       1),
                                        ('LCK11',       1),
                                        ('LCK12',       1),
                                        ('LCK13',       1),
                                        ('LCK14',       1),
                                        ('LCK15',       1),
                                        ('LCKK',        1),
                                        ]),
            Reg32 ('AFRL',      0x20,  [('AFSEL0',      4),
                                        ('AFSEL1',      4),
                                        ('AFSEL2',      4),
                                        ('AFSEL3',      4),
                                        ('AFSEL4',      4),
                                        ('AFSEL5',      4),
                                        ('AFSEL6',      4),
                                        ('AFSEL7',      4),
                                        ]),
            Reg32 ('AFRH',      0x24,  [('AFSEL8',      4),
                                        ('AFSEL9',      4),
                                        ('AFSEL10',     4),
                                        ('AFSEL11',     4),
                                        ('AFSEL12',     4),
                                        ('AFSEL13',     4),
                                        ('AFSEL14',     4),
                                        ('AFSEL15',     4),
                                        ]),
            Reg32 ('BRR',       0x28,  [('BR0',         1),
                                        ('BR1',         1),
                                        ('BR2',         1),
                                        ('BR3',         1),
                                        ('BR4',         1),
                                        ('BR5',         1),
                                        ('BR6',         1),
                                        ('BR7',         1),
                                        ('BR8',         1),
                                        ('BR9',         1),
                                        ('BR10',        1),
                                        ('BR11',        1),
                                        ('BR12',        1),
                                        ('BR13',        1),
                                        ('BR14',        1),
                                        ('BR15',        1),
                                        ]),
            ]

    def __init__(self, target, ap, name, addr, **kwargs):
        super(GPIO, self).__init__(target, ap, addr, name, GPIO.REGS, **kwargs)
