# Copyright (c) 2022 Phase Advanced Sensor Systems, Inc.
from ..device import Device, AReg32


class LPUART(Device):
    '''
    Driver for STM32G4 LPUART device (and probably compatible with others).
    '''
    REGS = [AReg32('CR1_FIFOEN',    0x000, [('UE',              0),
                                            ('UESM',            1),
                                            ('RE',              2),
                                            ('TE',              3),
                                            ('IDLEIE',          4),
                                            ('RXFNEIE',         5),
                                            ('TCIE',            6),
                                            ('TXFNFIE',         7),
                                            ('PEIE',            8),
                                            ('PS',              9),
                                            ('PCE',             10),
                                            ('WAKE',            11),
                                            ('M0',              12),
                                            ('MME',             13),
                                            ('CMIE',            14),
                                            ('DEDT',            16, 20),
                                            ('DEAT',            21, 25),
                                            ('M1',              28),
                                            ('FIFOEN',          29),
                                            ('TXFEIE',          30),
                                            ('RXFFIE',          31),
                                            ]),
            AReg32('CR1_FIFODIS',   0x000, [('UE',              0),
                                            ('UESM',            1),
                                            ('RE',              2),
                                            ('TE',              3),
                                            ('IDLEIE',          4),
                                            ('RXNEIE',          5),
                                            ('TCIE',            6),
                                            ('TXEIE',           7),
                                            ('PEIE',            8),
                                            ('PS',              9),
                                            ('PCE',             10),
                                            ('WAKE',            11),
                                            ('M0',              12),
                                            ('MME',             13),
                                            ('CMIE',            14),
                                            ('DEDT',            16, 20),
                                            ('DEAT',            21, 25),
                                            ('M1',              28),
                                            ('FIFOEN',          29),
                                            ]),
            AReg32('CR2',           0x004, [('ADDM7',           4),
                                            ('STOP',            12, 13),
                                            ('SWAP',            15),
                                            ('RXINV',           16),
                                            ('TXINV',           17),
                                            ('DATAINV',         18),
                                            ('MSBFIRST',        19),
                                            ('ADD',             24, 31),
                                            ]),
            AReg32('CR3',           0x008, [('EIE',             0),
                                            ('HDSEL',           3),
                                            ('DMAR',            6),
                                            ('DMAT',            7),
                                            ('RTSE',            8),
                                            ('CTSE',            9),
                                            ('CTSIE',           10),
                                            ('OVRDIS',          12),
                                            ('DDRE',            13),
                                            ('DEM',             14),
                                            ('DEP',             15),
                                            ('WUS',             20, 21),
                                            ('WUFIE',           22),
                                            ('TXFTIE',          23),
                                            ('RXFTCFG',         25, 27),
                                            ('RXFTIE',          28),
                                            ('TXFTCFG',         29, 31),
                                            ]),
            AReg32('BRR',           0x00C, [('BRR',             0,  19),
                                            ]),
            AReg32('RQR',           0x018, [('SBKRQ',           1),
                                            ('MMRQ',            2),
                                            ('RXFRQ',           3),
                                            ('TXFRQ',           4),
                                            ]),
            AReg32('ISR_FIFOEN',    0x01C, [('PE',              0),
                                            ('FE',              1),
                                            ('NE',              2),
                                            ('ORE',             3),
                                            ('IDLE',            4),
                                            ('RXFNE',           5),
                                            ('TC',              6),
                                            ('TXFNF',           7),
                                            ('CTSIF',           9),
                                            ('CTS',             10),
                                            ('BUSY',            16),
                                            ('CMF',             17),
                                            ('SBKF',            18),
                                            ('RWU',             19),
                                            ('WUF',             20),
                                            ('TEACK',           21),
                                            ('REACK',           22),
                                            ('TXFE',            23),
                                            ('RXFF',            24),
                                            ('RXFT',            26),
                                            ('TXFT',            27),
                                            ]),
            AReg32('ISR_FIFODIS',   0x01C, [('PE',              0),
                                            ('FE',              1),
                                            ('NE',              2),
                                            ('ORE',             3),
                                            ('IDLE',            4),
                                            ('RXNE',            5),
                                            ('TC',              6),
                                            ('TXE',             7),
                                            ('CTSIF',           9),
                                            ('CTS',             10),
                                            ('BUSY',            16),
                                            ('CMF',             17),
                                            ('SBKF',            18),
                                            ('RWU',             19),
                                            ('WUF',             20),
                                            ('TEACK',           21),
                                            ('REACK',           22),
                                            ]),
            AReg32('ICR',           0x020, [('PECF',            0),
                                            ('FECF',            1),
                                            ('NECF',            2),
                                            ('ORECF',           3),
                                            ('IDLECF',          4),
                                            ('TCCF',            6),
                                            ('CTSCF',           9),
                                            ('CMCF',            17),
                                            ('WUCF',            20),
                                            ]),
            AReg32('RDR',           0x024, [('RDR',             0, 8),
                                            ]),
            AReg32('TDR',           0x028, [('TDR',             0, 8),
                                            ]),
            AReg32('PRESC',         0x02C, [('PRESCALER',       0, 3),
                                            ]),
            ]

    def __init__(self, target, ap, name, addr, **kwargs):
        super().__init__(target, ap, addr, name, LPUART.REGS, **kwargs)
