# Copyright (c) 2022 Phase Advanced Sensor Systems, Inc.
from psdb.devices import Device, Reg8S, Reg32S, AReg32


class QUADSPI(Device):
    '''
    Driver for the STM32 quad-SPI peripheral found on many MCUs (QUADSPI).
    '''
    REGS = [AReg32('CR',        0x000, [('EN',           0),
                                        ('ABORT',        1),
                                        ('DMAEN',        2),
                                        ('TCEN',         3),
                                        ('SSHIFT',       4),
                                        ('DFM',          6),
                                        ('FSEL',         7),
                                        ('FTHRES',       8, 11),
                                        ('TEIE',        16),
                                        ('TCIE',        17),
                                        ('FTIE',        18),
                                        ('SMIE',        19),
                                        ('TOIE',        20),
                                        ('APMS',        22),
                                        ('PMM',         23),
                                        ('PRESCALER',   24, 31),
                                        ]),
            AReg32('DCR',       0x004, [('CKMODE',       0),
                                        ('CSHT',         8, 10),
                                        ('FSIZE',       16, 20),
                                        ]),
            AReg32('SR',        0x008, [('TEF',          0),
                                        ('TCF',          1),
                                        ('FTF',          2),
                                        ('SMF',          3),
                                        ('TOF',          4),
                                        ('BUSY',         5),
                                        ('FLEVEL',       8, 12),
                                        ]),
            AReg32('FCR',       0x00C, [('CTEF',         0),
                                        ('CTCF',         1),
                                        ('CSMF',         3),
                                        ('CTOF',         4),
                                        ]),
            AReg32('DLR',       0x010, [('DL',           0, 31),
                                        ]),
            AReg32('CCR',       0x014, [('INSTRUCTION',  0,  7),
                                        ('IMODE',        8,  9),
                                        ('AMODE',       10, 11),
                                        ('ADSIZE',      12, 13),
                                        ('ABMODE',      14, 15),
                                        ('ABSIZE',      16, 17),
                                        ('DCYC',        18, 22),
                                        ('DMODE',       24, 25),
                                        ('FMODE',       26, 27),
                                        ('SIOO',        28),
                                        ('DHHC',        30),
                                        ('DDRM',        31),
                                        ]),
            AReg32('AR',        0x018, [('ADDRESS',      0, 31),
                                        ]),
            AReg32('ABR',       0x01C, [('ALTERNATE',    0, 31),
                                        ]),
            Reg8S('DR8',        0x020),
            Reg32S('DR32',      0x020),
            AReg32('PSMKR',     0x024, [('MASK',         0, 31),
                                        ]),
            AReg32('PSMAR',     0x028, [('MATCH',        0, 31),
                                        ]),
            AReg32('PIR',       0x02C, [('INTERVAL',     0, 15),
                                        ]),
            AReg32('LPTR',      0x030, [('TIMEOUT',      0, 15),
                                        ]),
            ]

    def __init__(self, target, ap, name, addr, **kwargs):
        super().__init__(target, ap, addr, name, QUADSPI.REGS, **kwargs)
