# Copyright (c) 2020 Phase Advanced Sensor Systems, Inc.
from psdb.devices import Device, Reg32, AReg32, AReg32RS


class USB_HS(Device):
    '''
    Driver for the USB component found in the following MCUs:
        stm32h745
    '''
    REGS = [
        Reg32('GOTGCTL',    0x00,  [('SRQSCS',          1),
                                    ('SRQ',             1),
                                    ('VBVALOEN',        1),
                                    ('VBVALOVAL',       1),
                                    ('AVALOEN',         1),
                                    ('AVALOVAL',        1),
                                    ('BVALOEN',         1),
                                    ('BVALOVAL',        1),
                                    ('HNGSCS',          1),
                                    ('HNPRQ',           1),
                                    ('HSHNPEN',         1),
                                    ('DHNPEN',          1),
                                    ('EHEN',            1),
                                    ('',                3),
                                    ('CIDSTS',          1),
                                    ('DBCT',            1),
                                    ('ASVLD',           1),
                                    ('BSVLD',           1),
                                    ('OTGVER',          1),
                                    ('CURMOD',          1),
                                    ]),
        Reg32('GOTGINT',   0x04,   [('',                2),
                                    ('SEDET',           1),
                                    ('',                5),
                                    ('SRSSCHG',         1),
                                    ('HNSSCHG',         1),
                                    ('',                7),
                                    ('HNGDET',          1),
                                    ('ADTOCHG',         1),
                                    ('DBCDNE',          1),
                                    ]),
        Reg32('GAHBCFG',   0x08,   [('GINTMSK',         1),
                                    ('HBSTLEN',         4),
                                    ('DMAEN',           1),
                                    ('',                1),
                                    ('TXFELVL',         1),
                                    ('PTXFELVL',        1),
                                    ]),
        Reg32('GUSBCFG',   0x0C,   [('TOCAL',           3),
                                    ('',                3),
                                    ('PHYSEL',          1),
                                    ('',                1),
                                    ('SRPCAP',          1),
                                    ('HNPCAP',          1),
                                    ('TRDT',            4),
                                    ('',                1),
                                    ('PHYLPC',          1),
                                    ('',                1),
                                    ('ULPIFSL',         1),
                                    ('ULPIAR',          1),
                                    ('UPLICSM',         1),
                                    ('ULPIEVBUSD',      1),
                                    ('ULPIEVBUSI',      1),
                                    ('TSDPS',           1),
                                    ('PCCI',            1),
                                    ('PTCI',            1),
                                    ('ULPIIPD',         1),
                                    ('',                3),
                                    ('FHMOD',           1),
                                    ('FDMOD',           1),
                                    ]),
        AReg32('GRSTCTL',  0x010,  [('CSRST',               0),
                                    ('PSRST',               1),
                                    ('FCRST',               2),
                                    ('RXFLSH',              4),
                                    ('TXFLSH',              5),
                                    ('TXFNUM',              6,  10),
                                    ('DMAREQ',              30),
                                    ('AHBIDL',              31),
                                    ]),
        AReg32('GINTSTS',  0x014,  [('CMOD',                0),
                                    ('MMIS',                1),
                                    ('OTGINT',              2),
                                    ('SOF',                 3),
                                    ('RXFLVL',              4),
                                    ('NPTXFE',              5),
                                    ('GINAKEFF',            6),
                                    ('GONAKEFF',            7),
                                    ('ESUSP',               10),
                                    ('USBSUSP',             11),
                                    ('USBRST',              12),
                                    ('ENUMDNE',             13),
                                    ('ISOODRP',             14),
                                    ('EOPF',                15),
                                    ('IEPINT',              18),
                                    ('OEPINT',              19),
                                    ('IISOIXFR',            20),
                                    ('IPXFR/INCOMPISOOUT',  21),
                                    ('DATAFSUSP',           22),
                                    ('RSTDET',              23),
                                    ('HPRTINT',             24),
                                    ('HCINT',               25),
                                    ('PTXFE',               26),
                                    ('LPMINT',              27),
                                    ('CIDSCHG',             28),
                                    ('DISCINT',             29),
                                    ('SRQINT',              30),
                                    ('WKUPINT',             31),
                                    ]),
        AReg32('GINTMSK',  0x018,  [('MMISM',                   1),
                                    ('OTGINT',                  2),
                                    ('SOFM',                    3),
                                    ('RXFLVLM',                 4),
                                    ('NPTXFEM',                 5),
                                    ('GINAKEFFM',               6),
                                    ('GONAKEFFM',               7),
                                    ('ESUSPM',                  10),
                                    ('USBSUSPM',                11),
                                    ('USBRST',                  12),
                                    ('ENUMDNEM',                13),
                                    ('ISOODRPM',                14),
                                    ('EOPFM',                   15),
                                    ('IEPINT',                  18),
                                    ('OEPINT',                  19),
                                    ('IISOIXFRM',               20),
                                    ('IPXFRM/INCOMPISOOUTM',    21),
                                    ('FSUSPM',                  22),
                                    ('RSTDETM',                 23),
                                    ('PRTIM',                   24),
                                    ('HCIM',                    25),
                                    ('PTXFEM',                  26),
                                    ('LPMINTM',                 27),
                                    ('CIDSCHGM',                28),
                                    ('DISCINT',                 29),
                                    ('SRQIM',                   30),
                                    ('WUIM',                    31),
                                    ]),
        AReg32('GRXSTSR_DEV',      0x01C, [('EPNUM',       0,  3),
                                           ('BCNT',        4,  14),
                                           ('DPID',        15, 16),
                                           ('PKTSTS',      17, 20),
                                           ('FRMNUM',      21, 24),
                                           ('STSPHST',     27),
                                           ]),
        AReg32('GRXSTSR_HST',      0x01C, [('CHNUM',       0,  3),
                                           ('BCNT',        4,  14),
                                           ('DPID',        15, 16),
                                           ('PKTSTS',      17, 20),
                                           ]),
        AReg32RS('GRXSTSP_DEV',    0x020, [('EPNUM',       0,  3),
                                           ('BCNT',        4,  14),
                                           ('DPID',        15, 16),
                                           ('PKTSTS',      17, 20),
                                           ('FRMNUM',      21, 24),
                                           ('STSPHST',     27),
                                           ]),
        AReg32RS('GRXSTSP_HST',    0x020, [('CHNUM',       0,  3),
                                           ('BCNT',        4,  14),
                                           ('DPID',        15, 16),
                                           ('PKTSTS',      17, 20),
                                           ]),
        AReg32('GRXFSIZ',          0x024, [('RXFD',        0,  15),
                                           ]),
        AReg32('HNPTXFSIZ',        0x028, [('NPTXFSA',     0,  15),
                                           ('NPTXFD',      16, 31),
                                           ]),
        AReg32('DIEPTXF0',         0x028, [('TX0FSA',      0,  15),
                                           ('TX0FD',       16, 31),
                                           ]),
        AReg32('HNPTXSTS',         0x02C, [('NPTXFSAV',    0,  15),
                                           ('NPTQXSAV',    16, 23),
                                           ('NPTXQTOP',    24, 30),
                                           ]),
        AReg32('GCCFG',            0x038, [('DCDET',       0),
                                           ('PDET',        1),
                                           ('SDET',        2),
                                           ('PS2DET',      3),
                                           ('PWRDWN',      16),
                                           ('BCDEN',       17),
                                           ('DCDEN',       18),
                                           ('PDEN',        19),
                                           ('SDEN',        20),
                                           ('VBDEN',       21),
                                           ]),
        AReg32('CID',              0x03C, [('PRODUCT_ID',  0,  31),
                                           ]),
        AReg32('GLPMCFG',          0x054, [('LPMEN',       0),
                                           ('LPMACK',      1),
                                           ('BESL',        2,  5),
                                           ('REMWAKE',     6),
                                           ('L1SSEN',      7),
                                           ('BESLTHRS',    8,  11),
                                           ('L1DS',        12),
                                           ('LPMRSP',      13, 14),
                                           ('SLPSTS',      15),
                                           ('L1RSMOK',     16),
                                           ('LPMCHIDX',    17, 20),
                                           ('LPMCNT',      21, 23),
                                           ('SNDLPM',      24),
                                           ('LPMRCNTSTS',  25, 27),
                                           ('ENBESL',      28),
                                           ]),
        AReg32('HPTXFSIZ',          0x100, [('PTXSA',       0,  15),
                                            ('PTXFSIZ',     16, 31),
                                            ]),
        AReg32('DIEPTXF1',          0x104, [('INEPTXSA',    0,  15),
                                            ('INEPTXFD',    16, 31),
                                            ]),
        AReg32('DIEPTXF2',          0x108, [('INEPTXSA',    0,  15),
                                            ('INEPTXFD',    16, 31),
                                            ]),
        AReg32('DIEPTXF3',          0x10C, [('INEPTXSA',    0,  15),
                                            ('INEPTXFD',    16, 31),
                                            ]),
        AReg32('DIEPTXF4',          0x110, [('INEPTXSA',    0,  15),
                                            ('INEPTXFD',    16, 31),
                                            ]),
        AReg32('DIEPTXF5',          0x114, [('INEPTXSA',    0,  15),
                                            ('INEPTXFD',    16, 31),
                                            ]),
        AReg32('DIEPTXF6',          0x118, [('INEPTXSA',    0,  15),
                                            ('INEPTXFD',    16, 31),
                                            ]),
        AReg32('DIEPTXF7',          0x11C, [('INEPTXSA',    0,  15),
                                            ('INEPTXFD',    16, 31),
                                            ]),
        AReg32('DIEPTXF8',          0x120, [('INEPTXSA',    0,  15),
                                            ('INEPTXFD',    16, 31),
                                            ]),

        # HOST MODE REGISTERS WOULD GO HERE

        # DEVICE MODE registers.
        AReg32('DCFG',              0x800, [('DSPD',        0,  1),
                                            ('NZLSOHSK',    2),
                                            ('DAD',         4,  10),
                                            ('PFIVL',       11, 12),
                                            ('XCVRDLY',     14),
                                            ('ERRATIM',     15),
                                            ('DESCDMA',     23),
                                            ('PERSCHIVL',   24, 25),
                                            ]),
        AReg32('DCTL',              0x804, [('RWUSIG',      0),
                                            ('SDIS',        1),
                                            ('GINSTS',      2),
                                            ('GONSTS',      3),
                                            ('TCTL',        4,  6),
                                            ('SGINAK',      7),
                                            ('CGINAK',      8),
                                            ('SGONAK',      9),
                                            ('CGONAK',      10),
                                            ('POPRGDNE',    11),
                                            ('ENCONTONBNA', 17),
                                            ('DSBESLRJCT',  18),
                                            ]),
        AReg32('DSTS',              0x808, [('SUSPSTS',     0),
                                            ('ENUMSPD',     1,  2),
                                            ('EERR',        3),
                                            ('FNSOF',       8,  21),
                                            ('DEVLNSTS',    22, 23),
                                            ]),
        AReg32('DIEPMSK',           0x810, [('XFRCM',       0),
                                            ('EPDM',        1),
                                            ('AHBERRM',     2),
                                            ('TOM',         3),
                                            ('ITTXFEMSK',   4),
                                            ('INEPNMM',     5),
                                            ('INEPNEM',     6),
                                            ('TXFURM',      8),
                                            ('BNAM',        9),
                                            ('NAKM',        13),
                                            ]),
        AReg32('DOEPMSK',           0x814, [('XFRCM',       0),
                                            ('EPDM',        1),
                                            ('AHBERRM',     2),
                                            ('STUPM',       3),
                                            ('OTEPDM',      4),
                                            ('STSPHSRXM',   5),
                                            ('B2BSTSUPM',   6),
                                            ('OUTPKTERRM',  8),
                                            ('BNAM',        9),
                                            ('BERRM',       12),
                                            ('NAKMSK',      13),
                                            ('NYETMSK',     14),
                                            ]),
        AReg32('DAINT',             0x818, [('IEPINT',      0,  15),
                                            ('OEPINT',      16, 31),
                                            ]),
        AReg32('DAINTMSK',          0x81C, [('IEPM',        0,  15),
                                            ('OEPM',        16, 31),
                                            ]),
        AReg32('DVBUSDIS',          0x828, [('VBUSDT',      0,  15),
                                            ]),
        AReg32('DVBUSPULSE',        0x82C, [('DVBUSP',      0,  15),
                                            ]),
        AReg32('DTHRCTL',           0x830, [('NONISOTHREN', 0),
                                            ('ISOTHREN',    1),
                                            ('TXTHRLEN',    2,  10),
                                            ('RXTHREN',     16),
                                            ('RXTHRLEN',    17, 25),
                                            ('ARPEN',       27),
                                            ]),
        AReg32('DIEPEMPMSK',        0x834, [('INEPTXFEM',   0,  15),
                                            ]),
        AReg32('DEACHINT',          0x838, [('IEP1INT',     1),
                                            ('OEP1INT',     17),
                                            ]),
        AReg32('DEACHINTMSK',       0x83C, [('IEP1INTM',    1),
                                            ('OEP1INTM',    17),
                                            ]),
        AReg32('HS_DIEPEACHMSK1',   0x844, [('XFRCM',       0),
                                            ('EPDM',        1),
                                            ('AHBERRM',     2),
                                            ('TOM',         3),
                                            ('ITTXFEMSK',   4),
                                            ('INEPNEM',     6),
                                            ('TXFURM',      8),
                                            ('BNAM',        9),
                                            ('NAKM',        13),
                                            ]),
        AReg32('HS_DOEPEACHMSK1',   0x884, [('XFRCM',       0),
                                            ('EPDM',        1),
                                            ('AHBERRM',     2),
                                            ('STUPM',       3),
                                            ('OTEPDM',      4),
                                            ('B2BSTUPM',    6),
                                            ('OUTPKTERRM',  8),
                                            ('BNAM',        9),
                                            ('BERRM',       12),
                                            ('NAKMSK',      13),
                                            ('NYETMSK',     14),
                                            ]),

        # Per-endpoint device registers.
        AReg32('DIEPCTL0',          0x900, [('MPSIZ',           0,  10),
                                            ('USBAEP',          15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('STALL',           21),
                                            ('TXFNUM',          22, 25),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SODDFRM',         29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DIEPINT0',          0x908, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('TOC',             3),
                                            ('ITTXFE',          4),
                                            ('INEPNM',          5),
                                            ('INEPNE',          6),
                                            ('TXFE',            7),
                                            ('TXFIFOUDRN',      8),
                                            ('BNA',             9),
                                            ('PKTDRPSTS',       11),
                                            ('NAK',             13),
                                            ]),
        AReg32('DIEPTSIZ0',         0x910, [('XFRSIZ',          0,  6),
                                            ('PKTCNT',          19, 20),
                                            ]),
        AReg32('DIEPDMA0',          0x914, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DTXFSTS0',          0x918, [('INEPTFSAV',       0,  15),
                                            ]),
        AReg32('DIEPCTL1',          0x920, [('MPSIZ',           0,  10),
                                            ('USBAEP',          15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('STALL',           21),
                                            ('TXFNUM',          22, 25),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SODDFRM',         29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DIEPINT1',          0x928, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('TOC',             3),
                                            ('ITTXFE',          4),
                                            ('INEPNM',          5),
                                            ('INEPNE',          6),
                                            ('TXFE',            7),
                                            ('TXFIFOUDRN',      8),
                                            ('BNA',             9),
                                            ('PKTDRPSTS',       11),
                                            ('NAK',             13),
                                            ]),
        AReg32('DIEPTSIZ1',         0x930, [('XFRSIZ',          0,  18),
                                            ('PKTCNT',          19, 28),
                                            ('MCNT',            29, 30),
                                            ]),
        AReg32('DIEPDMA1',          0x934, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DTXFSTS1',          0x938, [('INEPTFSAV',       0,  15),
                                            ]),
        AReg32('DIEPCTL2',          0x940, [('MPSIZ',           0,  10),
                                            ('USBAEP',          15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('STALL',           21),
                                            ('TXFNUM',          22, 25),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SODDFRM',         29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DIEPINT2',          0x948, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('TOC',             3),
                                            ('ITTXFE',          4),
                                            ('INEPNM',          5),
                                            ('INEPNE',          6),
                                            ('TXFE',            7),
                                            ('TXFIFOUDRN',      8),
                                            ('BNA',             9),
                                            ('PKTDRPSTS',       11),
                                            ('NAK',             13),
                                            ]),
        AReg32('DIEPTSIZ2',         0x950, [('XFRSIZ',          0,  18),
                                            ('PKTCNT',          19, 28),
                                            ('MCNT',            29, 30),
                                            ]),
        AReg32('DIEPDMA2',          0x954, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DTXFSTS2',          0x958, [('INEPTFSAV',       0,  15),
                                            ]),
        AReg32('DIEPCTL3',          0x960, [('MPSIZ',           0,  10),
                                            ('USBAEP',          15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('STALL',           21),
                                            ('TXFNUM',          22, 25),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SODDFRM',         29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DIEPINT3',          0x968, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('TOC',             3),
                                            ('ITTXFE',          4),
                                            ('INEPNM',          5),
                                            ('INEPNE',          6),
                                            ('TXFE',            7),
                                            ('TXFIFOUDRN',      8),
                                            ('BNA',             9),
                                            ('PKTDRPSTS',       11),
                                            ('NAK',             13),
                                            ]),
        AReg32('DIEPTSIZ3',         0x970, [('XFRSIZ',          0,  18),
                                            ('PKTCNT',          19, 28),
                                            ('MCNT',            29, 30),
                                            ]),
        AReg32('DIEPDMA3',          0x974, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DTXFSTS3',          0x978, [('INEPTFSAV',       0,  15),
                                            ]),
        AReg32('DIEPCTL4',          0x980, [('MPSIZ',           0,  10),
                                            ('USBAEP',          15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('STALL',           21),
                                            ('TXFNUM',          22, 25),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SODDFRM',         29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DIEPINT4',          0x988, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('TOC',             3),
                                            ('ITTXFE',          4),
                                            ('INEPNM',          5),
                                            ('INEPNE',          6),
                                            ('TXFE',            7),
                                            ('TXFIFOUDRN',      8),
                                            ('BNA',             9),
                                            ('PKTDRPSTS',       11),
                                            ('NAK',             13),
                                            ]),
        AReg32('DIEPTSIZ4',         0x990, [('XFRSIZ',          0,  18),
                                            ('PKTCNT',          19, 28),
                                            ('MCNT',            29, 30),
                                            ]),
        AReg32('DIEPDMA4',          0x994, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DTXFSTS4',          0x998, [('INEPTFSAV',       0,  15),
                                            ]),
        AReg32('DIEPCTL5',          0x9A0, [('MPSIZ',           0,  10),
                                            ('USBAEP',          15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('STALL',           21),
                                            ('TXFNUM',          22, 25),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SODDFRM',         29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DIEPINT5',          0x9A8, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('TOC',             3),
                                            ('ITTXFE',          4),
                                            ('INEPNM',          5),
                                            ('INEPNE',          6),
                                            ('TXFE',            7),
                                            ('TXFIFOUDRN',      8),
                                            ('BNA',             9),
                                            ('PKTDRPSTS',       11),
                                            ('NAK',             13),
                                            ]),
        AReg32('DIEPTSIZ5',         0x9B0, [('XFRSIZ',          0,  18),
                                            ('PKTCNT',          19, 28),
                                            ('MCNT',            29, 30),
                                            ]),
        AReg32('DIEPDMA5',          0x9B4, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DTXFSTS5',          0x9B8, [('INEPTFSAV',       0,  15),
                                            ]),
        AReg32('DIEPCTL6',          0x9C0, [('MPSIZ',           0,  10),
                                            ('USBAEP',          15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('STALL',           21),
                                            ('TXFNUM',          22, 25),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SODDFRM',         29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DIEPINT6',          0x9C8, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('TOC',             3),
                                            ('ITTXFE',          4),
                                            ('INEPNM',          5),
                                            ('INEPNE',          6),
                                            ('TXFE',            7),
                                            ('TXFIFOUDRN',      8),
                                            ('BNA',             9),
                                            ('PKTDRPSTS',       11),
                                            ('NAK',             13),
                                            ]),
        AReg32('DIEPTSIZ6',         0x9D0, [('XFRSIZ',          0,  18),
                                            ('PKTCNT',          19, 28),
                                            ('MCNT',            29, 30),
                                            ]),
        AReg32('DIEPDMA6',          0x9D4, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DTXFSTS6',          0x9D8, [('INEPTFSAV',       0,  15),
                                            ]),
        AReg32('DIEPCTL7',          0x9E0, [('MPSIZ',           0,  10),
                                            ('USBAEP',          15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('STALL',           21),
                                            ('TXFNUM',          22, 25),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SODDFRM',         29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DIEPINT7',          0x9E8, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('TOC',             3),
                                            ('ITTXFE',          4),
                                            ('INEPNM',          5),
                                            ('INEPNE',          6),
                                            ('TXFE',            7),
                                            ('TXFIFOUDRN',      8),
                                            ('BNA',             9),
                                            ('PKTDRPSTS',       11),
                                            ('NAK',             13),
                                            ]),
        AReg32('DIEPTSIZ7',         0x9F0, [('XFRSIZ',          0,  18),
                                            ('PKTCNT',          19, 28),
                                            ('MCNT',            29, 30),
                                            ]),
        AReg32('DIEPDMA7',          0x9F4, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DTXFSTS7',          0x9F8, [('INEPTFSAV',       0,  15),
                                            ]),
        AReg32('DIEPCTL8',          0xA00, [('MPSIZ',           0,  10),
                                            ('USBAEP',          15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('STALL',           21),
                                            ('TXFNUM',          22, 25),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SODDFRM',         29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DIEPINT8',          0xA08, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('TOC',             3),
                                            ('ITTXFE',          4),
                                            ('INEPNM',          5),
                                            ('INEPNE',          6),
                                            ('TXFE',            7),
                                            ('TXFIFOUDRN',      8),
                                            ('BNA',             9),
                                            ('PKTDRPSTS',       11),
                                            ('NAK',             13),
                                            ]),
        AReg32('DIEPTSIZ8',         0xA10, [('XFRSIZ',          0,  18),
                                            ('PKTCNT',          19, 28),
                                            ('MCNT',            29, 30),
                                            ]),
        AReg32('DIEPDMA8',          0xA14, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DTXFSTS8',          0xA18, [('INEPTFSAV',       0,  15),
                                            ]),

        AReg32('DOEPCTL0',          0xB00, [('MPSIZ',           0,  1),
                                            ('USBAEP',          15),
                                            ('NAKSTS',          17),
                                            ('EPTYPE',          18, 19),
                                            ('SNPM',            20),
                                            ('STALL',           21),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DOEPINT0',          0xB08, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('STUP',            3),
                                            ('OTEPDIS',         4),
                                            ('STSPHSRX',        5),
                                            ('B2BSTUP',         6),
                                            ('OUTPKTERR',       8),
                                            ('BNA',             9),
                                            ('BERR',            12),
                                            ('NAK',             13),
                                            ('NYET',            14),
                                            ('STPKTRX',         15),
                                            ]),
        AReg32('DOEPTSIZ0',         0xB10, [('XFRSIZ',          0,  6),
                                            ('PKTCNT',          19),
                                            ('STUPCNT',         29, 30),
                                            ]),
        AReg32('DOEPDMA0',          0xB14, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DOEPCTL1',          0xB20, [('MPSIZ',           0,  10),
                                            ('USBA',            15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('SNPM',            20),
                                            ('STALL',           21),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SD1PID_SODDFRM',  29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DOEPINT1',          0xB28, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('STUP',            3),
                                            ('OTEPDIS',         4),
                                            ('STSPHSRX',        5),
                                            ('B2BSTUP',         6),
                                            ('OUTPKTERR',       8),
                                            ('BNA',             9),
                                            ('BERR',            12),
                                            ('NAK',             13),
                                            ('NYET',            14),
                                            ('STPKTRX',         15),
                                            ]),
        AReg32('DOEPTSIZ1',         0xB30, [('XFRSIZ',          0,  18),
                                            ('PKTNCT',          19, 28),
                                            ('RXDPID_STUPCNT',  29, 30),
                                            ]),
        AReg32('DOEPDMA1',          0xB34, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DOEPCTL2',          0xB40, [('MPSIZ',           0,  10),
                                            ('USBA',            15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('SNPM',            20),
                                            ('STALL',           21),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SD1PID_SODDFRM',  29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DOEPINT2',          0xB48, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('STUP',            3),
                                            ('OTEPDIS',         4),
                                            ('STSPHSRX',        5),
                                            ('B2BSTUP',         6),
                                            ('OUTPKTERR',       8),
                                            ('BNA',             9),
                                            ('BERR',            12),
                                            ('NAK',             13),
                                            ('NYET',            14),
                                            ('STPKTRX',         15),
                                            ]),
        AReg32('DOEPTSIZ2',         0xB50, [('XFRSIZ',          0,  18),
                                            ('PKTNCT',          19, 28),
                                            ('RXDPID_STUPCNT',  29, 30),
                                            ]),
        AReg32('DOEPDMA2',          0xB54, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DOEPCTL3',          0xB60, [('MPSIZ',           0,  10),
                                            ('USBA',            15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('SNPM',            20),
                                            ('STALL',           21),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SD1PID_SODDFRM',  29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DOEPINT3',          0xB68, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('STUP',            3),
                                            ('OTEPDIS',         4),
                                            ('STSPHSRX',        5),
                                            ('B2BSTUP',         6),
                                            ('OUTPKTERR',       8),
                                            ('BNA',             9),
                                            ('BERR',            12),
                                            ('NAK',             13),
                                            ('NYET',            14),
                                            ('STPKTRX',         15),
                                            ]),
        AReg32('DOEPTSIZ3',         0xB70, [('XFRSIZ',          0,  18),
                                            ('PKTNCT',          19, 28),
                                            ('RXDPID_STUPCNT',  29, 30),
                                            ]),
        AReg32('DOEPDMA3',          0xB74, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DOEPCTL4',          0xB80, [('MPSIZ',           0,  10),
                                            ('USBA',            15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('SNPM',            20),
                                            ('STALL',           21),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SD1PID_SODDFRM',  29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DOEPINT4',          0xB88, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('STUP',            3),
                                            ('OTEPDIS',         4),
                                            ('STSPHSRX',        5),
                                            ('B2BSTUP',         6),
                                            ('OUTPKTERR',       8),
                                            ('BNA',             9),
                                            ('BERR',            12),
                                            ('NAK',             13),
                                            ('NYET',            14),
                                            ('STPKTRX',         15),
                                            ]),
        AReg32('DOEPTSIZ4',         0xB90, [('XFRSIZ',          0,  18),
                                            ('PKTNCT',          19, 28),
                                            ('RXDPID_STUPCNT',  29, 30),
                                            ]),
        AReg32('DOEPDMA4',          0xB94, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DOEPCTL5',          0xBA0, [('MPSIZ',           0,  10),
                                            ('USBA',            15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('SNPM',            20),
                                            ('STALL',           21),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SD1PID_SODDFRM',  29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DOEPINT5',          0xBA8, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('STUP',            3),
                                            ('OTEPDIS',         4),
                                            ('STSPHSRX',        5),
                                            ('B2BSTUP',         6),
                                            ('OUTPKTERR',       8),
                                            ('BNA',             9),
                                            ('BERR',            12),
                                            ('NAK',             13),
                                            ('NYET',            14),
                                            ('STPKTRX',         15),
                                            ]),
        AReg32('DOEPTSIZ5',         0xBB0, [('XFRSIZ',          0,  18),
                                            ('PKTNCT',          19, 28),
                                            ('RXDPID_STUPCNT',  29, 30),
                                            ]),
        AReg32('DOEPDMA5',          0xBB4, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DOEPCTL6',          0xBC0, [('MPSIZ',           0,  10),
                                            ('USBA',            15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('SNPM',            20),
                                            ('STALL',           21),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SD1PID_SODDFRM',  29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DOEPINT6',          0xBC8, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('STUP',            3),
                                            ('OTEPDIS',         4),
                                            ('STSPHSRX',        5),
                                            ('B2BSTUP',         6),
                                            ('OUTPKTERR',       8),
                                            ('BNA',             9),
                                            ('BERR',            12),
                                            ('NAK',             13),
                                            ('NYET',            14),
                                            ('STPKTRX',         15),
                                            ]),
        AReg32('DOEPTSIZ6',         0xBD0, [('XFRSIZ',          0,  18),
                                            ('PKTNCT',          19, 28),
                                            ('RXDPID_STUPCNT',  29, 30),
                                            ]),
        AReg32('DOEPDMA6',          0xBD4, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DOEPCTL7',          0xBE0, [('MPSIZ',           0,  10),
                                            ('USBA',            15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('SNPM',            20),
                                            ('STALL',           21),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SD1PID_SODDFRM',  29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DOEPINT7',          0xBE8, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('STUP',            3),
                                            ('OTEPDIS',         4),
                                            ('STSPHSRX',        5),
                                            ('B2BSTUP',         6),
                                            ('OUTPKTERR',       8),
                                            ('BNA',             9),
                                            ('BERR',            12),
                                            ('NAK',             13),
                                            ('NYET',            14),
                                            ('STPKTRX',         15),
                                            ]),
        AReg32('DOEPTSIZ7',         0xBF0, [('XFRSIZ',          0,  18),
                                            ('PKTNCT',          19, 28),
                                            ('RXDPID_STUPCNT',  29, 30),
                                            ]),
        AReg32('DOEPDMA7',          0xBF4, [('DMAADDR',         0,  31),
                                            ]),
        AReg32('DOEPCTL8',          0xC00, [('MPSIZ',           0,  10),
                                            ('USBA',            15),
                                            ('EONUM_DPID',      16),
                                            ('NAKSTS',          17),
                                            ('EPTYP',           18, 19),
                                            ('SNPM',            20),
                                            ('STALL',           21),
                                            ('CNAK',            26),
                                            ('SNAK',            27),
                                            ('SD0PID_SEVNFRM',  28),
                                            ('SD1PID_SODDFRM',  29),
                                            ('EPDIS',           30),
                                            ('EPENA',           31),
                                            ]),
        AReg32('DOEPINT8',          0xC08, [('XFRC',            0),
                                            ('EPDISD',          1),
                                            ('AHBERR',          2),
                                            ('STUP',            3),
                                            ('OTEPDIS',         4),
                                            ('STSPHSRX',        5),
                                            ('B2BSTUP',         6),
                                            ('OUTPKTERR',       8),
                                            ('BNA',             9),
                                            ('BERR',            12),
                                            ('NAK',             13),
                                            ('NYET',            14),
                                            ('STPKTRX',         15),
                                            ]),
        AReg32('DOEPTSIZ8',         0xC10, [('XFRSIZ',          0,  18),
                                            ('PKTNCT',          19, 28),
                                            ('RXDPID_STUPCNT',  29, 30),
                                            ]),
        AReg32('DOEPDMA8',          0xC14, [('DMAADDR',         0,  31),
                                            ]),

        AReg32('PCGCCTL',           0xE00, [('STPPCLK',         0),
                                            ('GATEHCLK',        1),
                                            ('PHYSUSP',         4),
                                            ('ENL1GTG',         5),
                                            ('PHYSLEEP',        6),
                                            ('SUSP',            7),
                                            ]),
        ]

    def __init__(self, target, ap, name, addr, **kwargs):
        super().__init__(target, ap, addr, name, USB_HS.REGS, **kwargs)
