# Copyright (c) 2020 Phase Advanced Sensor Systems, Inc.
from ..device import Device, Reg32


class SYSCFG(Device):
    '''
    Driver for the STM32G4 SYSCFG device.
    '''
    REGS = [Reg32('MEMRMP', 0x000, [('MEM_MODE', 3),
                                    ('',         5),
                                    ('FB_MODE',  1),
                                    ]),
            Reg32('CFGR1',  0x004, [('',            8),
                                    ('BOOSTEN',     1),
                                    ('ANASWVDD',    1),
                                    ('',            6),
                                    ('I2C_PB6_FMP', 1),
                                    ('I2C_PB7_FMP', 1),
                                    ('I2C_PB8_FMP', 1),
                                    ('I2C_PB9_FMP', 1),
                                    ('I2C1_FMP',    1),
                                    ('I2C2_FMP',    1),
                                    ('I2C3_FMP',    1),
                                    ('I2C4_FMP',    1),
                                    ('',            2),
                                    ('FPU_IE',      6),
                                    ]),
            Reg32('EXTICR1', 0x008, [('EXTI0',      4),
                                     ('EXTI1',      4),
                                     ('EXTI2',      4),
                                     ('EXTI3',      4),
                                     ]),
            Reg32('EXTICR2', 0x00C, [('EXTI4',      4),
                                     ('EXTI5',      4),
                                     ('EXTI6',      4),
                                     ('EXTI7',      4),
                                     ]),
            Reg32('EXTICR3', 0x010, [('EXTI8',      4),
                                     ('EXTI9',      4),
                                     ('EXTI10',     4),
                                     ('EXTI11',     4),
                                     ]),
            Reg32('EXTICR4', 0x014, [('EXTI12',     4),
                                     ('EXTI13',     4),
                                     ('EXTI14',     4),
                                     ('EXTI15',     4),
                                     ]),
            Reg32('SCSR',    0x018, [('CCMER',      1),
                                     ('CCMBSY',     1),
                                     ]),
            Reg32('CFGR2',   0x01C, [('CLL',        1),
                                     ('SPL',        1),
                                     ('PVDL',       1),
                                     ('ECCL',       1),
                                     ('',           4),
                                     ('SPF',        1),
                                     ]),
            Reg32('SWPR',    0x020, [('P0WP',       1),
                                     ('P1WP',       1),
                                     ('P2WP',       1),
                                     ('P3WP',       1),
                                     ('P4WP',       1),
                                     ('P5WP',       1),
                                     ('P6WP',       1),
                                     ('P7WP',       1),
                                     ('P8WP',       1),
                                     ('P9WP',       1),
                                     ('P10WP',      1),
                                     ('P11WP',      1),
                                     ('P12WP',      1),
                                     ('P13WP',      1),
                                     ('P14WP',      1),
                                     ('P15WP',      1),
                                     ('P16WP',      1),
                                     ('P17WP',      1),
                                     ('P18WP',      1),
                                     ('P19WP',      1),
                                     ('P20WP',      1),
                                     ('P21WP',      1),
                                     ('P22WP',      1),
                                     ('P23WP',      1),
                                     ('P24WP',      1),
                                     ('P25WP',      1),
                                     ('P26WP',      1),
                                     ('P27WP',      1),
                                     ('P28WP',      1),
                                     ('P29WP',      1),
                                     ('P30WP',      1),
                                     ('P31WP',      1),
                                     ]),
            Reg32('SKR',      0x24, [('KEY',        8),
                                     ]),
            ]

    def __init__(self, target, ap, name, addr, **kwargs):
        super(SYSCFG, self).__init__(target, ap, addr, name, SYSCFG.REGS,
                                     **kwargs)
