# Copyright (c) 2020 Phase Advanced Sensor Systems, Inc.
from ..device import Device, Reg32


class SYSCFG(Device):
    '''
    Driver for the STM System Configuration (SYSCFG) device.
    '''
    REGS = [Reg32('PMCR',       0x04,  [('I2C1FMP',         1),
                                        ('I2C2FMP',         1),
                                        ('I2C3FMP',         1),
                                        ('I2C4FMP',         1),
                                        ('PB6FMP',          1),
                                        ('PB7FMP',          1),
                                        ('PB8FMP',          1),
                                        ('PB9FMP',          1),
                                        ('BOOSTE',          1),
                                        ('BOOSTVDDSEL',     1),
                                        ('',                11),
                                        ('EPIS',            3),
                                        ('PA0SO',           1),
                                        ('PA1SO',           1),
                                        ('PA2SO',           1),
                                        ('PA3SO',           1),
                                        ]),
            Reg32('EXTICR1',    0x08,  [('EXTI0',           4),
                                        ('EXTI1',           4),
                                        ('EXTI2',           4),
                                        ('EXTI3',           4),
                                        ]),
            Reg32('EXTICR2',    0x0C,  [('EXTI4',           4),
                                        ('EXTI5',           4),
                                        ('EXTI6',           4),
                                        ('EXTI7',           4),
                                        ]),
            Reg32('EXTICR3',    0x10,  [('EXTI8',           4),
                                        ('EXTI9',           4),
                                        ('EXTI10',          4),
                                        ('EXTI11',          4),
                                        ]),
            Reg32('EXTICR4',    0x14,  [('EXTI12',          4),
                                        ('EXTI13',          4),
                                        ('EXTI14',          4),
                                        ('EXTI15',          4),
                                        ]),
            Reg32('CFGR',       0x18,  [('CM4L',            1),
                                        ('',                1),
                                        ('PVDL',            1),
                                        ('FLASHL',          1),
                                        ('',                2),
                                        ('CM7L',            1),
                                        ('BKRAML',          1),
                                        ('',                1),
                                        ('SRAM4L',          1),
                                        ('SRAM3L',          1),
                                        ('SRAM2L',          1),
                                        ('SRAM1L',          1),
                                        ('DTCML',           1),
                                        ('ITCML',           1),
                                        ('AXISRAML',        1),
                                        ]),
            Reg32('CCCSR',      0x20,  [('EN',              1),
                                        ('CS',              1),
                                        ('',                6),
                                        ('READY',           1),
                                        ('',                7),
                                        ('HSLV',            1),
                                        ]),
            Reg32('CCVR',       0x24,  [('NCV',             4),
                                        ('PCV',             4),
                                        ]),
            Reg32('CCCR',       0x28,  [('NCC',             4),
                                        ('PCC',             4),
                                        ]),
            Reg32('PWRCR',      0x2C,  [('ODEN',            1),
                                        ]),
            Reg32('PKGR',       0x124, [('PKG',             4),
                                        ]),
            Reg32('UR0',        0x300, [('BKS',             1),
                                        ('',                15),
                                        ('RDP',             8),
                                        ]),
            Reg32('UR1',        0x304, [('BCM4',            1),
                                        ('',                15),
                                        ('BCM7',            1),
                                        ]),
            Reg32('UR2',        0x308, [('BORH',            2),
                                        ('',                14),
                                        ('BCM7_ADD0',       16),
                                        ]),
            Reg32('UR3',        0x30C, [('BCM4_ADD0',       16),
                                        ('BCM7_ADD1',       16),
                                        ]),
            Reg32('UR4',        0x310, [('BCM4_ADD1',       16),
                                        ('MEPAD_1',         1),
                                        ]),
            Reg32('UR5',        0x314, [('MESAD_1',         1),
                                        ('',                15),
                                        ('WRPS_1',          8),
                                        ]),
            Reg32('UR6',        0x318, [('PA_BEG_1',        12),
                                        ('',                4),
                                        ('PA_END_1',        12),
                                        ]),
            Reg32('UR7',        0x31C, [('SA_BEG_1',        12),
                                        ('',                4),
                                        ('SA_END_1',        12),
                                        ]),
            Reg32('UR8',        0x320, [('MEPAD_2',         1),
                                        ('',                15),
                                        ('MESAD_2',         1),
                                        ]),
            Reg32('UR9',        0x324, [('WRPS_2',          8),
                                        ('',                8),
                                        ('PA_BEG_2',        12),
                                        ]),
            Reg32('UR10',       0x328, [('PA_END_2',        12),
                                        ('',                4),
                                        ('SA_BEG_2',        12),
                                        ]),
            Reg32('UR11',       0x32C, [('SA_END_2',        12),
                                        ('',                4),
                                        ('IWDG1M',          1),
                                        ]),
            Reg32('UR12',       0x330, [('IWDG2M',          1),
                                        ('',                15),
                                        ('SECURE',          1),
                                        ]),
            Reg32('UR13',       0x334, [('SDRS',            2),
                                        ('',                14),
                                        ('D1SBRST',         1),
                                        ]),
            Reg32('UR14',       0x338, [('D1STPRST',        1),
                                        ('',                15),
                                        ('D2BRST',          1),
                                        ]),
            Reg32('UR15',       0x33C, [('D2STPRST',        1),
                                        ('',                15),
                                        ('FZIWDGSTB',       1),
                                        ]),
            Reg32('UR16',       0x340, [('FZIWDGSTP',       1),
                                        ('',                15),
                                        ('PKP',             1),
                                        ]),
            Reg32('UR17',       0x344, [('IO_HSLV',         1),
                                        ]),
            ]

    def __init__(self, target, ap, name, addr, **kwargs):
        super(SYSCFG, self).__init__(target, ap, addr, name, SYSCFG.REGS,
                                     **kwargs)
